<?php
/**
 * Wonder Grid setup
 *
 * @package Wonder Grid
 * @since   1.0.0
 */

defined( 'ABSPATH' ) || exit;

/**
 * Main Wonder_Grid Class.
 *
 * @class Wonder_Grid
 */
final class Wonder_Grid {

	/**
	 * Wonder Grid version.
	 *
	 * @var string
	 */
	public $version = '1.0.7';

	/**
	 * The single instance of the class.
	 *
	 * @var Wonder_Grid
	 * @since 1.0.0
	 */
	protected static $_instance = null;

	/**
	 * The arguments of wonder-grid post type
	 *
	 * @var Wonder_Grid
	 * @since 1.0.0
	 */
	public $post_type = 'wonder-grid';
	public $post_type_slug = 'project';
	public $post_type_description;
	public $post_type_supports = array( 'title', 'editor', 'thumbnail' );
	public $post_type_labels;
	public $post_type_label;
	public $public = true;
	public $publicly_queryable = true;
	public $show_ui = true;
	public $show_in_menu = true;
	public $query_var = true;
	public $rewrite = true;
	public $capability_type = 'post';
	public $has_archive = true;
	public $hierarchical = false;
	public $menu_position = null;
	public $exclude_from_search = false;
	public $show_in_nav_menus = true;
	public $show_in_admin_bar = true;
	public $menu_icon = 'dashicons-layout';
	public $map_meta_cap = null;
	public $can_export = true;
	public $show_in_rest = true;

	/**
	 * The arguments of wonder-grid taxonomy
	 *
	 * @var Wonder_Grid
	 * @since 1.0.0
	 */
	public $register_taxonomy = true;
	public $taxonomy   = 'wonder-grid-type';
	public $taxonomy_slug = 'project-type';
	public $taxonomy_labels;
	public $taxonomy_label;
	public $taxonomy_public = true;
	public $taxonomy_publicly_queryable = true;
	public $taxonomy_show_ui = true;
	public $taxonomy_show_in_menu = true;
	public $taxonomy_show_in_nav_menus = true;
	public $taxonomy_show_tagcloud = false;
	public $taxonomy_show_in_rest = true;
	public $taxonomy_show_in_quick_edit = true;
	public $taxonomy_show_admin_column = true;
	public $taxonomy_query_var = true;

	/**
	 * Main Wonder_Grid Instance.
	 *
	 * Ensures only one instance of Wonder_Grid is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 * @static
	 * @see Wonder_Grid()
	 * @return Wonder_Grid - Main instance.
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Cloning is forbidden.
	 *
	 * @since 1.0.0
	 */
	public function __clone() {
		wc_doing_it_wrong( __FUNCTION__, __( 'Cloning is forbidden.', 'wonder-grid' ), '1.0.3' );
	}

	/**
	 * Unserializing instances of this class is forbidden.
	 *
	 * @since 1.0.0
	 */
	public function __wakeup() {
		wc_doing_it_wrong( __FUNCTION__, __( 'Unserializing instances of this class is forbidden.', 'wonder-grid' ), '1.0.3' );
	}

	/**
	 * SuperPro Constructor.
	 */
	public function __construct() {

		$this->define_constants();
		$this->includes();
		$this->init_hooks();

		add_image_size( 'wonder-grid-columns-thumbnail', 150, 150 );
		add_image_size( 'wonder-grid-columns-2', 800, 800 );
		add_image_size( 'wonder-grid-columns-3', 600, 600 );
		add_image_size( 'wonder-grid-columns-4', 400, 400 );
		add_image_size( 'wonder-grid-columns-5', 300, 300 );

	    $this->post_type_labels = array(
	    	'name'               => esc_html__( 'Projects', 'wonder-grid' ),
			'singular_name'      => esc_html__( 'Project', 'wonder-grid' ),
			'menu_name'          => esc_html__( 'Wonder Grid', 'wonder-grid' ),
			'name_admin_bar'     => esc_html__( 'Wonder Grid', 'wonder-grid' ),
			'add_new'            => esc_html__( 'Add New', 'wonder-grid' ),
			'add_new_item'       => esc_html__( 'Add New Project', 'wonder-grid' ),
			'new_item'           => esc_html__( 'New Project', 'wonder-grid' ),
			'edit_item'          => esc_html__( 'Edit Project', 'wonder-grid' ),
			'view_item'          => esc_html__( 'View Project', 'wonder-grid' ),
			'all_items'          => esc_html__( 'All Projects', 'wonder-grid' ),
			'search_items'       => esc_html__( 'Search Projects', 'wonder-grid' ),
			'parent_item_colon'  => esc_html__( 'Parent Projects:', 'wonder-grid' ),
			'not_found'          => esc_html__( 'No project found.', 'wonder-grid' ),
			'not_found_in_trash' => esc_html__( 'No project found in Trash.', 'wonder-grid' )
	    );
		$this->post_type_label = esc_html__( 'Portfolio', 'wonder-grid' );

		$this->taxonomy_labels = array(
	    	'name'               => esc_html__( 'Project Types', 'wonder-grid' ),
			'singular_name'      => esc_html__( 'Project Type', 'wonder-grid' ),
			'search_items'       => esc_html__( 'Search Project Types', 'wonder-grid' ),
			'all_items'          => esc_html__( 'All Project Types', 'wonder-grid' ),
			'parent_item'        => esc_html__( 'Parent Project Types', 'wonder-grid' ),
			'parent_item_colon'  => esc_html__( 'Parent Project Type:', 'wonder-grid' ),
			'edit_item'          => esc_html__( 'Edit Project Type', 'wonder-grid' ),
			'update_item'        => esc_html__( 'Update Project Type', 'wonder-grid' ),
			'add_new_item'       => esc_html__( 'Add New Project Type', 'wonder-grid' ),
			'new_item_name'      => esc_html__( 'New Genre Project Type', 'wonder-grid' ),
			'menu_name'          => esc_html__( 'Project Types', 'wonder-grid' ),
	    );
		$this->taxonomy_label = esc_html__( 'Project Types', 'wonder-grid' );
	}

	public function settings( $post_type_slug, $taxonomy_slug ){
		$this->post_type_slug = $post_type_slug;
		$this->taxonomy_slug = $taxonomy_slug;
	}

	/**
	 * Init Hooks
	 */
	public function init_hooks(){
		add_action( 'init', array( $this,'textdomain' ) );
		add_action( 'init', array( $this, 'post_type' ) );
		add_action( 'init', array( $this, 'taxonomy' ), 0 );
		add_action( 'wp_enqueue_scripts', array( $this, 'scripts' ), 30 );
		add_filter( 'single_template', array( $this, 'project_single_template') );
		add_filter( 'template_include', array( $this, 'project_taxonomy_template' ) );
		add_filter( 'template_include', array( $this, 'project_grid_template' ) );
		add_filter( 'register_post_type_args', array( $this, 'post_type_args'), 10, 2 );
		add_filter( 'register_taxonomy_args',  array( $this, 'taxonomy_args' ), 10, 2 );
		add_action( 'elementor/elements/categories_registered', array( $this, 'new_elementor_category' ) );
		add_action( 'elementor/widgets/widgets_registered', array( $this, 'widgets_registered' ) );
		add_action( 'init',	array( $this, 'check_update' ) );
		add_action( 'after_switch_theme', array( $this, 'elementor_global_option' ) );
	}

	/**
	 * Define Wonder Grid Constants.
	 */
	private function define_constants() {
		$this->define( 'WG_DEBUG', true );
		$this->define( 'WG_ABSPATH', dirname( WONDER_GRID_FILE ) . '/' );
		$this->define( 'WG_PLUGIN_BASENAME', plugin_basename( WONDER_GRID_FILE ) );
		$this->define( 'WG_DIR', plugin_dir_path( dirname(__FILE__) ) );
		$this->define( 'WG_DIR_URI', plugins_url( __FILE__ ) );
    $this->define( 'WG_ROOT_DIR', plugins_url().'/'.plugin_basename( WG_DIR ).'/' );
		$this->define( 'WG_VERSION', $this->version );
	}

	/**
	 * Define constant if not already set.
	 *
	 * @param string      $name  Constant name.
	 * @param string|bool $value Constant value.
	 */
	private function define( $name, $value ) {
		if ( ! defined( $name ) ) {
			define( $name, $value );
		}
	}

	/**
     * Localize
     */
	public function textdomain() {
	   load_plugin_textdomain( 'wonder-grid', FALSE, dirname( plugin_basename( WONDER_GRID_FILE ) ).'/languages/' );
	}

	/**
	 * Register Portfolio Post Type
	 */
	public function post_type() {
		/**
		 * Register a book post type.
		 *
		 * @link http://codex.wordpress.org/Function_Reference/register_post_type
		 */

		$args = apply_filters( 'wonder_grid_post_type_args', array(
			'labels'             => $this->post_type_labels,
			'label'				       => $this->post_type_label,
	    'description'        => $this->post_type_description,
			'public'             => $this->public,
			'publicly_queryable' => $this->publicly_queryable,
			'show_ui'            => $this->show_ui,
			'show_in_menu'       => $this->show_in_menu,
			'show_in_nav_menus'  => $this->show_in_nav_menus,
			'show_in_admin_bar'  => $this->show_in_admin_bar,
			'query_var'          => $this->query_var,
			'capability_type'    => $this->capability_type,
			'has_archive'        => $this->has_archive,
			'hierarchical'       => $this->hierarchical,
			'menu_position'      => $this->menu_position,
			'menu_icon'			     => $this->menu_icon,
			'supports'           => $this->post_type_supports,
			'exclude_from_search'=> $this->exclude_from_search,
			'map_meta_cap'		   => $this->map_meta_cap,
			'can_export'		     => $this->can_export,
			'show_in_rest'	  	 => $this->show_in_rest,
			'rest_base'          => $this->post_type,
    	'rest_controller_class' => 'WP_REST_Posts_Controller',
			'taxonomies'		 => array( $this->taxonomy )
		));

		if( $this->rewrite ){
			$args['rewrite'] = array( 'slug' => $this->post_type_slug );
		}else{
			$args['rewrite'] = $this->rewrite;
		}

		register_post_type( $this->post_type, $args );
	}

	/**
	 * Register Portfolio Category Taxonomy
	 */
	public function taxonomy() {
		if(!$this->register_taxonomy){
			return;
		}

		$args = apply_filters( 'wonder_grid_taxonomy_labels', array(
			'hierarchical'       => true,
			'label'              => $this->taxonomy_label,
			'labels'             => $this->taxonomy_labels,
			'public'             => $this->taxonomy_public,
			'publicly_queryable' => $this->taxonomy_publicly_queryable,
			'show_ui'            => $this->taxonomy_show_ui,
			'show_in_menu'       => $this->taxonomy_show_in_menu,
			'show_in_nav_menus'  => $this->taxonomy_show_in_nav_menus,
			'show_admin_column'  => $this->taxonomy_show_admin_column,
			'show_tagcloud'      => $this->taxonomy_show_tagcloud,
			'show_in_rest'  	 => $this->taxonomy_show_in_rest,
			'rest_base'          => $this->taxonomy,
    		'rest_controller_class' => 'WP_REST_Terms_Controller',
			'show_in_quick_edit' => $this->taxonomy_show_in_quick_edit,
			'query_var'  		 => $this->taxonomy_query_var,
			'rewrite'            => array( 'slug' => $this->taxonomy_slug ),
		) );

		register_taxonomy( $this->taxonomy, $this->post_type, $args );
		register_taxonomy_for_object_type( $this->taxonomy, $this->post_type );
	}

	/**
	 * Add REST API support to an already registered post type.
	 */
	public function post_type_args( $args, $post_type ) {

	    if ( $this->post_type === $post_type ) {
	        $args['show_in_rest'] = true;

	        // Optionally customize the rest_base or rest_controller_class
	        $args['rest_base']             = $this->post_type;
	        $args['rest_controller_class'] = 'WP_REST_Posts_Controller';
	    }

	    return $args;
	}

	/**
	 * Add REST API support to an already registered taxonomy.
	 */
	public function taxonomy_args( $args, $taxonomy_name ) {

	    if ( $this->taxonomy === $taxonomy_name ) {
	        $args['show_in_rest'] = true;

	        // Optionally customize the rest_base rest_controller_class
	        $args['rest_base']             =  $this->taxonomy;
	        $args['rest_controller_class'] = 'WP_REST_Terms_Controller';
	    }

	    return $args;
	}

	/**
	 * The single post template
	 */
	public function project_single_template( $single ) {

	    global $post;

	    $single_template_file = 'single-project.php';

	    /* Checks for single template by post type */
	    if ( get_post_type( $post->ID ) == $this->post_type ) {
	    	if( file_exists( trailingslashit( get_stylesheet_directory() ) . 'wonder-grid/' . $single_template_file ) ) {
	    		 return trailingslashit( get_stylesheet_directory() ) . 'wonder-grid/' . $single_template_file;
	    	}else{
		        if ( file_exists( WG_ABSPATH . 'templates/' . $single_template_file ) ) {
		            return WG_ABSPATH . 'templates/' . $single_template_file;
		        } else {
		        	return new WP_Error( 'broke', esc_html__( "The project post template doesn't exist.", "wonder-grid" ) );
		        }
	    	}
	    }else{
	    	return $single;
	    }
	}


	/**
	 * The taxonomy template
	 */
	public function project_taxonomy_template( $template ){

		$taxonomy_template_file = 'taxonomy-project-type.php';

	    if( is_tax( $this->taxonomy ) ) {
	        if( file_exists( trailingslashit( get_stylesheet_directory() ) . 'wonder-grid/' . $taxonomy_template_file ) ) {
	            $template = trailingslashit( get_stylesheet_directory()) . 'wonder-grid/' . $taxonomy_template_file;
	        } else {
	        	if ( file_exists( WG_ABSPATH . 'templates/' . $taxonomy_template_file ) ) {
	            	$template = WG_ABSPATH . 'templates/' . $taxonomy_template_file;
	        	}else{
	        		return new WP_Error( 'broke', esc_html__( "The project taxonomy template doesn't exist.", "wonder-grid" ) );
	        	}
	        }
	    }

	    return $template;
	}

	/**
	 * The protfolio grid page template
	 */
	public function project_grid_template( $template ){

		$grid_page_template_file = 'wonder-grid-page-template.php';

	    if( is_page_template( 'wonder-grid-page-template' ) ) {
	        if( file_exists( trailingslashit( get_stylesheet_directory() ) . 'wonder-grid/' . $grid_page_template_file ) ) {
	            $template = trailingslashit( get_stylesheet_directory()) . 'wonder-grid/' . $grid_page_template_file;
	        } else {
	        	if ( file_exists( WG_ABSPATH . 'templates/' . $grid_page_template_file ) ) {
	            	$template = WG_ABSPATH . 'templates/' . $grid_page_template_file;
	        	}else{
	        		return new WP_Error( 'broke', esc_html__( "The project grid page template doesn't exist.", "wonder-grid" ) );
	        	}
	        }
	    }

	    return $template;
	}

	/**
	 * Styles / Scripts
	 */
	public function scripts() {

		global $post;

		$min = ! WG_DEBUG ? 'min.' : '';

		wp_enqueue_style( 'wonder-grid', plugins_url( 'assets/css/wonder-grid.' . esc_html( $min ) . 'css', WONDER_GRID_FILE ), '', null );
		if( is_single() && get_post_type( $post->ID ) == $this->post_type ) {
			wp_enqueue_style( 'swiper', plugins_url( 'assets/vendors/swiper/swiper.min.css', WONDER_GRID_FILE ), '', null );
			wp_enqueue_script( 'swiper', plugins_url( 'assets/vendors/swiper/swiper.min.js', WONDER_GRID_FILE ), array('jquery'), null, true );
			wp_enqueue_script( 'jquery-sticky', plugins_url( 'assets/vendors/jquery-sticky.js', WONDER_GRID_FILE ), array('jquery'), null, true );
		}

		wp_enqueue_script( 'isotope', plugins_url( 'assets/vendors/isotope.pkgd.min.js', WONDER_GRID_FILE ), array('jquery'), null, true );
		wp_enqueue_script( 'lazyload', plugins_url( 'assets/vendors/lazyload.min.js', WONDER_GRID_FILE ), array('jquery'), null, true );
		wp_enqueue_script( 'infinite-scroll', plugins_url( 'assets/vendors/infinite-scroll.pkgd.min.js', WONDER_GRID_FILE ), array('jquery'), null, true );
		wp_enqueue_script( 'wonder-grid', plugins_url( 'assets/js/wonder-grid.' . esc_html( $min ) . 'js', WONDER_GRID_FILE ), array( 'jquery', 'imagesloaded' ), null, true );
	}

	/**
     * New Elementor Category
     */
    public function new_elementor_category($elements_manager) {
        $elements_manager->add_category(
            'wonder-grid',
            array(
                'title' => esc_html__( 'Wonder Grid', 'wonder-grid' ),
                'icon' => 'fa fa-plug',
            ),
        2 );
   }

	/**
	 * Register Widget
	 */
    public function widgets_registered( $widgets_manager ) {

      include_once WG_ABSPATH . 'includes/builder/wonder-grid-widget.php';
      $widgets_manager->register_widget_type( new Wonder_Grid_Widget() );

			include_once WG_ABSPATH . 'includes/builder/wonder-grid-blog.php';
      $widgets_manager->register_widget_type( new Wonder_Grid_Blog_Widget() );

    }

	/**
	 * Include required core files used in admin and on the frontend.
	 */
	public function includes() {
		require_once WG_ABSPATH . 'includes/class-template-loader.php';
		require_once WG_ABSPATH . 'includes/functions.php';
		require_once WG_ABSPATH . 'includes/class-plugin-page-template-loader.php';
		require_once WG_ABSPATH . 'includes/class-page-template.php';
		require_once WG_ABSPATH . 'includes/metabox/init.php';
		require_once WG_ABSPATH . 'includes/post-metabox.php';
		require_once WG_ABSPATH . 'includes/page-metabox.php';
		require_once WG_ABSPATH . 'includes/type-metabox.php';
		require_once WG_ABSPATH . 'includes/options.php';
		require_once WG_ABSPATH . 'includes/builder/wonder-grid-wpb.php';
		require_once WG_ABSPATH . 'includes/plugin-update-checker/plugin-update-checker.php';
	}

	/**
	 * Load Template
	 */
    public function load_template( $slug, $name = '', $settings = array() ) {

        $templates = new WG_Template_Loader();
        $data = array();

        foreach( $settings as $key => $value ) {
            $data[$key] = $value;
        }

        $templates
           ->set_template_data( $data, 'wg_data' )
           ->get_template_part( $slug, $name );
    }

    /**
     * Get page setting
     */
    public function page_setting( $setting ) {
    	if( ! is_page() ) {
    		return;
    	}

    	global $post;
    	return get_post_meta( $post->ID, '_wonder_grid_' . $setting, true );
    }

    /**
     * Get project type setting
     */
    public function type_setting( $setting ) {
    	if( ! is_tax() ) {
    		return;
    	}

    	$term_id = get_queried_object()->term_id;
    	return get_term_meta( $term_id, '_wonder_grid_' . $setting, true );
    }

    /**
	 * Update global options to avoid theme conflict
	 */
	public function elementor_global_option () {
	  update_option('elementor_cpt_support', array( 'page', 'post', 'download', 'product', 'wonder-grid' ) );
	}

    public function check_update(){
			$UpdateChecker = Puc_v4_Factory::buildUpdateChecker(
				'https://focuxtheme.com/update-server/?action=get_metadata&slug=wonder-grid',
				WONDER_GRID_FILE,
				'wonder-grid'
			);
	}

}
