<?php
/**
 * The plugin functions.
 * All functions are pluggable, so you can copy it to your functions.php and make changes.
 * @version 1.0.0
 * @package Wonder Grid
 */

/**
 * Load Template Part
 */
if( ! function_exists( 'wg_load_template' ) ) {
	function wg_load_template( $slug, $name = '', $settings = array() ){
		$GLOBALS['Wonder_Grid']->load_template( $slug, $name, $settings );
	}
}

/**
 * Taxonomy Filter
 */
if( ! function_exists( 'wg_taxonomy_filter' ) ) {
    function wg_taxonomy_filter( $id, $types, $output = true ) {

    	if( $output == 0 || ! $output ) {
    		return;
    	}

	    $project_types = is_array( $types ) ? implode( ',', $types ) : '';

    	$terms = get_terms( array(
          'taxonomy' => 'wonder-grid-type',
          'hide_empty' => true,
          'parent' => 0,
          'include' => $project_types
        ) );

    	if ( ! isset( $terms ) || ! is_array( $terms ) ) {
    		return;
    	}

    	echo '<nav id="wg-filter-' . esc_attr( $id ) . '" class="wg-filter" data-option-key="filter">
    		    <ul>
			  	    <li class="wg-filter-active"><a href="javascript:;" data-filter="*">'. esc_html__( 'All', 'wonder-grid' ) .'</a></li>';
			        foreach( $terms as $term ) {
			        	echo '<li><a href="javascript:void(0);" data-filter=".wg-' . esc_attr( $term->slug ) . '">' . esc_html( $term->name ) . '</a></li>';
			        }
		echo '	</ul>
        	  </nav>';
    }
}

/**
 * Show the project media
 */
if( ! function_exists( 'wg_media' ) ) {
	function wg_media(){

		global $post;

		$media_type = get_post_meta( $post->ID, '_wonder_grid_media_type', true );
		$media_images_layout = get_post_meta( $post->ID, '_wonder_grid_images_layout', true );
	    $media_images = get_post_meta( $post->ID, '_wonder_grid_images', true );
	    $media_video = get_post_meta( $post->ID, '_wonder_grid_video_url', true );
	    $media_audio = get_post_meta( $post->ID, '_wonder_grid_audio_url', true );

	    switch( $media_type ) {

	  	  case 'images':

	  	    if( isset( $media_images ) ){

	  	    	if( $media_images_layout === 'slider' ) {
		  	    	echo '<div class="wg-slider">
		  	    		<div class="swiper-container">
		  	    			<div class="swiper-wrapper">';
					foreach ( (array) $media_images as $image_id => $image_url ) {
						echo '<div class="swiper-slide media-item">' . wp_get_attachment_image( $image_id, 'large' ) . '</div>';
					}
					echo '</div>

						  <!-- If we need pagination -->
	    				  <div class="swiper-pagination"></div>
						</div>

	    				<!-- If we need navigation buttons -->
						<div class="swiper-button-prev"></div>
						<div class="swiper-button-next"></div>
					  </div>';
				} else {
					foreach ( (array) $media_images as $image_id => $image_url ) {
					   echo '<div class="media-item">' . wp_get_attachment_image( $image_id, 'large' ) . '</div>';
				    }
				}
			}
			break;

		  case 'video':
		    if ( strpos( $media_video, 'youtube' ) > 0 || strpos( $media_video, 'youtu.be' ) > 0 ) {
			    $pattern = '%(?:youtube(?:-nocookie)?\.com/(?:[^/]+/.+/|(?:v|e(?:mbed)?)/|.*[?&]v=)|youtu\.be/)([^"&?/ ]{11})%i';
			  	$video_id = preg_match($pattern, $media_video, $match) ? $match[1] : '';
			  	echo '<div class="wg-media"><iframe width="560" height="315" src="https://youtube.com/embed/' . $video_id . '" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe></div>';
			    break;
			} else {
				$id = array();
				$video_id = '';
				if ( preg_match("/https?:\/\/(?:www\.)?vimeo.com\/(?:channels\/(?:\w+\/)?|groups\/([^\/]*)\/videos\/|album\/(\d+)\/video\/|)(\d+)(?:$|\/|\?)/", $media_video, $id)) {
					 $video_id = $id[3];
				}
				echo '<div class="wg-media"><iframe src="https://player.vimeo.com/video/' . $video_id . '" width="640" height="360" frameborder="0" allow="autoplay; fullscreen" allowfullscreen></iframe></div>';
				break;
			}

		  case 'audio':
		    echo '<iframe scrolling="no" width="100%" height="450" frameborder="no" src="https://w.soundcloud.com/player/?url=' . esc_url( $media_audio ) . '&auto_play=false&hide_related=false&show_comments=false&show_user=false&show_reposts=false&show_teaser=true&visual=true"></iframe>';
		    break;

		  case 'content':
		  	the_content();
		  	break;

	    }
	}
}
add_action( 'wonder_grid/project_media', 'wg_media' );

/**
 * Show the project meta
 */
if( ! function_exists( 'wg_meta' ) ) {
	function wg_meta() {
		echo '<p class="wg-meta">' . esc_html__( 'Project Type: ', 'wonder-grid' ) . wg_taxonomy( get_the_ID() ) .'</p>';
	}
}


/**
 * Show the project taxonomy
 */
if( ! function_exists( 'wg_taxonomy' ) ) {
	function wg_taxonomy( $id ) {
	   $terms = get_the_terms( $id, 'wonder-grid-type' );
       if ( ! empty( $terms ) ) {
		  foreach ( $terms as $term ) {
		  	if( $term->parent == 0 ) {
		      $url = get_term_link( $term->slug, 'wonder-grid-type' );
		      echo '<a href="' . esc_url( $url ) . '" class="wg-taxonomy">' . esc_html( $term->name ) . '</a>';
		      foreach( $terms as $sub_term ) {
		         if( $sub_term->parent == $term->term_id ) {
		         	 $sub_url = get_term_link( $sub_term->slug, 'wonder-grid-type' );
		         	 echo '» <a href="' . esc_url( $sub_url ) . '" class="wg-taxonomy child">' . esc_html( $sub_term->name ) . '</a>';
		         }
		      }
		    }
		  }
	   }
	}
}

/**
 * Return the project type slugs of a post.
 */
if( ! function_exists( 'wg_taxonomy_slug' ) ) {
	function wg_taxonomy_slug() {
		global $post;

		$terms = get_the_terms( $post->ID, 'wonder-grid-type' );
		$slug = array();

		if ( ! empty( $terms ) ) {
			if ( $terms && ! is_wp_error( $terms ) ){
				foreach( $terms as $term ) {
					if( $term->parent == 0 ) {
				      $slug[] = 'wg-' . $term->slug;
				      foreach( $terms as $sub_term ) {
				         if( $sub_term->parent == $term->term_id ) {
				         	$slug[] = 'wg-' . $sub_term->slug;
				         }
				      }
				    }
				}
			}
		}

		return join( " ", $slug);
	}
}

/**
 * Return the project types list.
 */
if( ! function_exists( 'wg_taxonomy_list' ) ) {
	function wg_taxonomy_list() {
		  $taxonomy_list = array();

		  $taxonomy_query = get_terms( array(
		    'taxonomy'   => 'wonder-grid-type',
		    'hide_empty' => false //you can change this if you need, chech get_terms() documentation
		  ));

		  foreach ( $taxonomy_query as $taxonomy_term ) {
		    // creates an array of 'term-slug' => 'Term Name'
		    $taxonomy_list[$taxonomy_term->slug] = $taxonomy_term->name;
		  }

		  return $taxonomy_list;
	}
}

/**
 * Show the project information
 */
if( ! function_exists( 'wg_info' ) ) {
	function wg_info(){

		global $post;

		$client = get_post_meta( $post->ID, '_wonder_grid_client_name', true );
		$url = get_post_meta( $post->ID, '_wonder_grid_project_url', true );
		$skills = get_post_meta( $post->ID, '_wonder_grid_skills_needed', true );
		$date = get_post_meta( $post->ID, '_wonder_grid_finish_date', true );

		$info = array(
	  	  'client' => array(
	  	  	  esc_html__( 'Client Name', 'wonder-grid' ), $client
	  	  ),
	  	  'project_url' => array(
	  	  	  esc_html__( 'Project URL', 'wonder-grid' ), $url
	  	  ),
	  	  'skills' => array(
	  	 	  esc_html__( 'Skills Needed', 'wonder-grid' ), $skills
	  	  ),
	  	  'date' => array(
	  	  	  esc_html__( 'Finish Date', 'wonder-grid' ), $date
	  	  )
	    );

	    //$info = apply_filters( 'wonder_grid/project_info', $info );

		foreach( $info as $key => $val ){
			if( $val[1] !== '' ) {
				if( strpos( $val[1], 'http://' ) === 0 || strpos( $val[1], 'https://' ) === 0 ){
				  echo '<li><strong>' . esc_html( $val[0] ) . ':</strong> <a href="' .esc_url( $val[1] ). '" target="_blank" rel="nofollow">' . esc_url( $val[1] ) . '</a></li>';
				}else{
				  echo '<li><strong>' . esc_html( $val[0] ) . ':</strong> ' . esc_html( $val[1] ) . '</li>';
				}
			}
	    }
	}
}
add_action( 'wonder_grid/project_info', 'wg_info' );

/**
 * The project grid items makeup.
 */
if( ! function_exists( 'wg_grid_item' ) ){
	function wg_grid_item( $args ){
		wg_load_template( 'content-grid', 'item', $args );
	}
}
add_action( 'wonder_grid/grid_items', 'wg_grid_item' );

/**
 * The project blog items makeup.
 */
if( ! function_exists( 'wg_blog_item' ) ){
	function wg_blog_item( $args ){
		wg_load_template( 'content-grid-blog', 'item', $args );
	}
}
add_action( 'wonder_grid/blog_items', 'wg_blog_item' );


/**
 * Register Shortcode
 */
if( ! function_exists('wg_grid_shortcode') ) {
	function wg_grid_shortcode($atts){

	   	$params = array(
	      'number'		   => 9,
	      'orderby'        => 'date',
	      'order'          => 'desc',
	      'pagination'	   => 0,
	      'infinite'	   => 0,
	      'columns'		   => 3,
	      'title_position' => 'outside',
	      'hover_animation'=> 'no-animation',
	      'filter'		   => 0,
	      'categories'     => ''
	    );

	    $data = shortcode_atts( $params, $atts );

	    ob_start();
	   	wg_load_template( 'wonder-grid', 'widget', $data );
	    return ob_get_clean();
	}
}
add_shortcode( 'wonder_grid', 'wg_grid_shortcode' );

/**
 * The wrapper of single project content
 * Hook: wonder_grid_before_single_content
 */
if( ! function_exists( 'wg_wrapper_start' ) ) {
	function wg_wrapper_start() {
		echo '<div id="content" class="site-content">';
	}
}
add_action( 'wonder_grid/before_single_content', 'wg_wrapper_start' );
add_action( 'wonder_grid/tax_wrapper_start', 'wg_wrapper_start' );
add_action( 'wonder_grid/grid_template_wrapper_start', 'wg_wrapper_start' );

/**
 * The wrapper close tag of single project content
 * Hook: wonder_grid_after_single_content
 */
if( ! function_exists( 'wg_wrapper_close' ) ) {
	function wg_wrapper_close() {
		echo '</div>';
	}
}
add_action( 'wonder_grid/after_single_content', 'wg_wrapper_close' );
add_action( 'wonder_grid/tax_wrapper_close', 'wg_wrapper_close' );
add_action( 'wonder_grid/grid_template_wrapper_close', 'wg_wrapper_close' );


/**
 * Single Post Navigation
 */
if( ! function_exists( 'wg_single_post_navi') ) {
	function wg_single_post_navi() {

		if( ! wg_get_option( 'wonder-grid', 'prev_next_post' ) ) {
			return;
		}

		$taxonomy = 'wonder-grid-type';
?>
		<div class="wg-single-post-navi">
		  <?php
		  $prevPost = get_previous_post( true, '', $taxonomy );
		  if( $prevPost ):
		  ?>
		  <div class="nav-box previous">
		  <?php $prevthumbnail = get_the_post_thumbnail( $prevPost->ID, array( 500,500 ) );?>
		  <?php previous_post_link( '%link','<span class="post-thumbnail">' . $prevthumbnail . '<h4 class="post-title">&larr; %title</h4></span>', TRUE, '', $taxonomy); ?>
		  </div>

		  <?php
		  endif;

		  $nextPost = get_next_post( true, '', $taxonomy );
		  if( $nextPost ):
		  ?>
		  <div class="nav-box next" style="float:right;">
		  <?php $nextthumbnail = get_the_post_thumbnail( $nextPost->ID, array( 500,500 ) ); ?>
		  <?php next_post_link( '%link', '<span class="post-thumbnail">' . $nextthumbnail . '<h4 class="post-title">%title &rarr;</h4></span>', TRUE, '', $taxonomy ); ?>
		  </div>
		  <?php endif; ?>
		</div>
		<div class="clear"></div>
<?php
	}
}

/**
 * Pagination
 */
if( ! function_exists( 'wg_pagination' ) ) {
	function wg_pagination() {
		$prev_text = sprintf(
			'%s <span class="nav-prev-text">%s</span>',
			'<span aria-hidden="true">&larr;</span>',
			__( 'Previous', 'wonder-grid' )
		);
		$next_text = sprintf(
			'<span class="nav-next-text">%s</span> %s',
			__( 'Next', 'wonder-grid' ),
			'<span aria-hidden="true">&rarr;</span>'
		);

		$posts_pagination = get_the_posts_pagination(
			array(
				'mid_size'  => 1,
				'prev_text' => $prev_text,
				'next_text' => $next_text,
			)
		);

		// If we're not outputting the previous page link, prepend a placeholder with visibility: hidden to take its place.
		if ( strpos( $posts_pagination, 'prev page-numbers' ) === false ) {
			$posts_pagination = str_replace( '<div class="nav-links">', '<div class="nav-links"><span class="prev page-numbers placeholder" aria-hidden="true">' . $prev_text . '</span>', $posts_pagination );
		}

		// If we're not outputting the next page link, append a placeholder with visibility: hidden to take its place.
		if ( strpos( $posts_pagination, 'next page-numbers' ) === false ) {
			$posts_pagination = str_replace( '</div>', '<span class="next page-numbers placeholder" aria-hidden="true">' . $next_text . '</span></div>', $posts_pagination );
		}

		if ( $posts_pagination ) { ?>

			<div class="wonder-grid-pagenavi section-inner">

				<?php echo $posts_pagination; //phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped -- already escaped during generation. ?>

			</div><!-- .pagination-wrapper -->

			<?php
		}
	}
}
